@@ -42,7 +42,6 @@
  * DO_UNFILLED:  Decompose triangles to lines and points where appropriate.
  * DO_TWOSTENCIL:Gross hack for two-sided stencil.
  *
- * HAVE_RGBA: Vertices have rgba values (otherwise index values).
  * HAVE_SPEC: Vertices have secondary rgba values.
  *
  * VERT_X(v): Alias for vertex x value.
@@ -59,29 +58,15 @@
  * VERT_COPY_RGBA: Copy vertex rgba another vertex.
  * VERT_SAVE_RGBA: Save vertex rgba to a local variable.
  * VERT_RESTORE_RGBA: Restore vertex rgba from a local variable.
- *   --> Similar for IND and SPEC.
+ *   --> Similar for SPEC.
  *
  * LOCAL_VARS(n): (At least) define local vars for save/restore rgba.
  *
  */
 
-#if HAVE_RGBA
-#define VERT_SET_IND( v, c ) (void) c
-#define VERT_COPY_IND( v0, v1 )
-#define VERT_SAVE_IND( idx )
-#define VERT_RESTORE_IND( idx )
 #if HAVE_BACK_COLORS
 #define VERT_SET_RGBA( v, c )
 #endif
-#else
-#define VERT_SET_RGBA( v, c ) (void) c
-#define VERT_COPY_RGBA( v0, v1 )
-#define VERT_SAVE_RGBA( idx )
-#define VERT_RESTORE_RGBA( idx )
-#if HAVE_BACK_COLORS
-#define VERT_SET_IND( v, c )
-#endif
-#endif
 
 #if !HAVE_SPEC
 #define VERT_SET_SPEC( v, c ) (void) c
@@ -99,7 +84,6 @@
 
 #if !HAVE_BACK_COLORS
 #define VERT_COPY_SPEC1( v )
-#define VERT_COPY_IND1( v )
 #define VERT_COPY_RGBA1( v )
 #endif
 
@@ -132,7 +116,7 @@ static void TAG(triangle)( GLcontext *ctx, GLuint e0, GLuint e1, GLuint e2 )
    struct vertex_buffer *VB = &TNL_CONTEXT( ctx )->vb;
    VERTEX *v[3];
    GLfloat offset = 0;
-   GLfloat z[3];
+   GLfloat z[3] = { 0 };
    GLenum mode = GL_FILL;
    GLuint facing = 0;
    LOCAL_VARS(3);
@@ -171,81 +155,67 @@ static void TAG(triangle)( GLcontext *ctx, GLuint e0, GLuint e1, GLuint e2 )
 	    }
 	 }
 
-	 if (DO_TWOSIDE && facing == 1)
-	 {
-	    if (HAVE_RGBA) {
-	       if (HAVE_BACK_COLORS) {
-		  if (!DO_FLAT) {
-		     VERT_SAVE_RGBA( 0 );
-		     VERT_SAVE_RGBA( 1 );
-		     VERT_COPY_RGBA1( v[0] );
-		     VERT_COPY_RGBA1( v[1] );
-		  }
-		  VERT_SAVE_RGBA( 2 );
-		  VERT_COPY_RGBA1( v[2] );
-		  if (HAVE_SPEC) {
-		     if (!DO_FLAT) {
-			VERT_SAVE_SPEC( 0 );
-			VERT_SAVE_SPEC( 1 );
-			VERT_COPY_SPEC1( v[0] );
-			VERT_COPY_SPEC1( v[1] );
-		     }
-		     VERT_SAVE_SPEC( 2 );
-		     VERT_COPY_SPEC1( v[2] );
-		  }
+	 if (DO_TWOSIDE && facing == 1) {
+	    if (HAVE_BACK_COLORS) {
+	       if (!DO_FLAT) {
+		  VERT_SAVE_RGBA( 0 );
+		  VERT_SAVE_RGBA( 1 );
+		  VERT_COPY_RGBA1( v[0] );
+		  VERT_COPY_RGBA1( v[1] );
 	       }
-	       else {
-		  GLfloat (*vbcolor)[4] = VB->ColorPtr[1]->data;
-		  (void) vbcolor;
-
+	       VERT_SAVE_RGBA( 2 );
+	       VERT_COPY_RGBA1( v[2] );
+	       if (HAVE_SPEC) {
 		  if (!DO_FLAT) {
-		     VERT_SAVE_RGBA( 0 );
-		     VERT_SAVE_RGBA( 1 );
-		  }
-		  VERT_SAVE_RGBA( 2 );
-
-		  if (VB->ColorPtr[1]->stride) {
-		     ASSERT(VB->ColorPtr[1]->stride == 4*sizeof(GLfloat));
-
-		     if (!DO_FLAT) {		  
-			VERT_SET_RGBA( v[0], vbcolor[e0] );
-			VERT_SET_RGBA( v[1], vbcolor[e1] );
-		     }
-		     VERT_SET_RGBA( v[2], vbcolor[e2] );
-		  }
-		  else {
-		     if (!DO_FLAT) {		  
-			VERT_SET_RGBA( v[0], vbcolor[0] );
-			VERT_SET_RGBA( v[1], vbcolor[0] );
-		     }
-		     VERT_SET_RGBA( v[2], vbcolor[0] );
-		  }
-
-		  if (HAVE_SPEC && VB->SecondaryColorPtr[1]) {
-		     GLfloat (*vbspec)[4] = VB->SecondaryColorPtr[1]->data;
-		     ASSERT(VB->SecondaryColorPtr[1]->stride == 4*sizeof(GLfloat));
-
-		     if (!DO_FLAT) {
-			VERT_SAVE_SPEC( 0 );
-			VERT_SAVE_SPEC( 1 );
-			VERT_SET_SPEC( v[0], vbspec[e0] );
-			VERT_SET_SPEC( v[1], vbspec[e1] );
-		     }
-		     VERT_SAVE_SPEC( 2 );
-		     VERT_SET_SPEC( v[2], vbspec[e2] );
+		     VERT_SAVE_SPEC( 0 );
+		     VERT_SAVE_SPEC( 1 );
+		     VERT_COPY_SPEC1( v[0] );
+		     VERT_COPY_SPEC1( v[1] );
 		  }
+		  VERT_SAVE_SPEC( 2 );
+		  VERT_COPY_SPEC1( v[2] );
 	       }
 	    }
 	    else {
-	       GLfloat (*vbindex) = (GLfloat *)VB->IndexPtr[1]->data;
+	       GLfloat (*vbcolor)[4] = VB->BackfaceColorPtr->data;
+	       (void) vbcolor;
+
 	       if (!DO_FLAT) {
-		  VERT_SAVE_IND( 0 );
-		  VERT_SAVE_IND( 1 );
-		  VERT_SET_IND( v[0], vbindex[e0] );
-		  VERT_SET_IND( v[1], vbindex[e1] );
+		  VERT_SAVE_RGBA( 0 );
+		  VERT_SAVE_RGBA( 1 );
+	       }
+	       VERT_SAVE_RGBA( 2 );
+
+	       if (VB->BackfaceColorPtr->stride) {
+		  ASSERT(VB->BackfaceColorPtr->stride == 4*sizeof(GLfloat));
+
+		  if (!DO_FLAT) {
+		     VERT_SET_RGBA( v[0], vbcolor[e0] );
+		     VERT_SET_RGBA( v[1], vbcolor[e1] );
+		  }
+		  VERT_SET_RGBA( v[2], vbcolor[e2] );
+	       }
+	       else {
+		  if (!DO_FLAT) {
+		     VERT_SET_RGBA( v[0], vbcolor[0] );
+		     VERT_SET_RGBA( v[1], vbcolor[0] );
+		  }
+		  VERT_SET_RGBA( v[2], vbcolor[0] );
+	       }
+
+	       if (HAVE_SPEC && VB->BackfaceSecondaryColorPtr) {
+		  GLfloat (*vbspec)[4] = VB->BackfaceSecondaryColorPtr->data;
+		  ASSERT(VB->BackfaceSecondaryColorPtr->stride == 4*sizeof(GLfloat));
+
+		  if (!DO_FLAT) {
+		     VERT_SAVE_SPEC( 0 );
+		     VERT_SAVE_SPEC( 1 );
+		     VERT_SET_SPEC( v[0], vbspec[e0] );
+		     VERT_SET_SPEC( v[1], vbspec[e1] );
+		  }
+		  VERT_SAVE_SPEC( 2 );
+		  VERT_SET_SPEC( v[2], vbspec[e2] );
 	       }
-	       VERT_SAVE_IND( 2 );
-	       VERT_SET_IND( v[2], vbindex[e2] );
 	    }
 	 }
       }
@@ -267,30 +237,22 @@ static void TAG(triangle)( GLcontext *ctx, GLuint e0, GLuint e1, GLuint e2 )
 	    GLfloat bc	= b * ic;
 	    if ( ac < 0.0f ) ac = -ac;
 	    if ( bc < 0.0f ) bc = -bc;
-	    offset += MAX2( ac, bc ) * ctx->Polygon.OffsetFactor;
+	    offset += MAX2( ac, bc ) * ctx->Polygon.OffsetFactor / ctx->DrawBuffer->_MRD;
 	 }
 	 offset *= ctx->DrawBuffer->_MRD * (REVERSE_DEPTH ? -1.0 : 1.0);
       }
    }
 
    if (DO_FLAT) {
-      if (HAVE_RGBA) {
-	 VERT_SAVE_RGBA( 0 );
-	 VERT_SAVE_RGBA( 1 );
-	 VERT_COPY_RGBA( v[0], v[2] );
-	 VERT_COPY_RGBA( v[1], v[2] );
-	 if (HAVE_SPEC && VB->SecondaryColorPtr[0]) {
-	    VERT_SAVE_SPEC( 0 );
-	    VERT_SAVE_SPEC( 1 );
-	    VERT_COPY_SPEC( v[0], v[2] );
-	    VERT_COPY_SPEC( v[1], v[2] );
-	 }
-      }
-      else {
-	 VERT_SAVE_IND( 0 );
-	 VERT_SAVE_IND( 1 );
-	 VERT_COPY_IND( v[0], v[2] );
-	 VERT_COPY_IND( v[1], v[2] );
+      VERT_SAVE_RGBA( 0 );
+      VERT_SAVE_RGBA( 1 );
+      VERT_COPY_RGBA( v[0], v[2] );
+      VERT_COPY_RGBA( v[1], v[2] );
+      if (HAVE_SPEC && VB->AttribPtr[_TNL_ATTRIB_COLOR1]) {
+	 VERT_SAVE_SPEC( 0 );
+	 VERT_SAVE_SPEC( 1 );
+	 VERT_COPY_SPEC( v[0], v[2] );
+	 VERT_COPY_SPEC( v[1], v[2] );
       }
    }
 
@@ -344,58 +306,42 @@ static void TAG(triangle)( GLcontext *ctx, GLuint e0, GLuint e1, GLuint e2 )
       VERT_SET_Z(v[2], z[2]);
    }
 
-   if (DO_TWOSIDE && facing == 1)
-   {
-      if (HAVE_RGBA) {
-	 if (!DO_FLAT) {
-	    VERT_RESTORE_RGBA( 0 );
-	    VERT_RESTORE_RGBA( 1 );
-	 }
-	 VERT_RESTORE_RGBA( 2 );
-	 if (HAVE_SPEC) {
-	    if (!DO_FLAT) {
-	       VERT_RESTORE_SPEC( 0 );
-	       VERT_RESTORE_SPEC( 1 );
-	    }
-	    VERT_RESTORE_SPEC( 2 );
-	 }
+   if (DO_TWOSIDE && facing == 1) {
+      if (!DO_FLAT) {
+	 VERT_RESTORE_RGBA( 0 );
+	 VERT_RESTORE_RGBA( 1 );
       }
-      else {
+      VERT_RESTORE_RGBA( 2 );
+      if (HAVE_SPEC) {
 	 if (!DO_FLAT) {
-	    VERT_RESTORE_IND( 0 );
-	    VERT_RESTORE_IND( 1 );
+	    VERT_RESTORE_SPEC( 0 );
+	    VERT_RESTORE_SPEC( 1 );
 	 }
-	 VERT_RESTORE_IND( 2 );
+	 VERT_RESTORE_SPEC( 2 );
       }
    }
 
 
    if (DO_FLAT) {
-      if (HAVE_RGBA) {
 	 VERT_RESTORE_RGBA( 0 );
 	 VERT_RESTORE_RGBA( 1 );
-	 if (HAVE_SPEC && VB->SecondaryColorPtr[0]) {
+	 if (HAVE_SPEC && VB->AttribPtr[_TNL_ATTRIB_COLOR1]) {
 	    VERT_RESTORE_SPEC( 0 );
 	    VERT_RESTORE_SPEC( 1 );
 	 }
-      }
-      else {
-	 VERT_RESTORE_IND( 0 );
-	 VERT_RESTORE_IND( 1 );
-      }
    }
 }
 #endif
 
 #if DO_QUAD
 #if DO_FULL_QUAD
-static void TAG(quad)( GLcontext *ctx,
+static void TAG(quadr)( GLcontext *ctx,
 		       GLuint e0, GLuint e1, GLuint e2, GLuint e3 )
 {
    struct vertex_buffer *VB = &TNL_CONTEXT( ctx )->vb;
    VERTEX *v[4];
    GLfloat offset = 0;
-   GLfloat z[4];
+   GLfloat z[4] = { 0 };
    GLenum mode = GL_FILL;
    GLuint facing = 0;
    LOCAL_VARS(4);
@@ -433,90 +379,74 @@ static void TAG(quad)( GLcontext *ctx,
 	    }
 	 }
 
-	 if (DO_TWOSIDE && facing == 1)
-	 {
-	    if (HAVE_RGBA) {
-	       GLfloat (*vbcolor)[4] = VB->ColorPtr[1]->data;
-	       (void)vbcolor;
+	 if (DO_TWOSIDE && facing == 1) {
+	    GLfloat (*vbcolor)[4] = VB->BackfaceColorPtr->data;
+	    (void)vbcolor;
 
-	       if (HAVE_BACK_COLORS) {
-                  if (!DO_FLAT) {
-                     VERT_SAVE_RGBA( 0 );
-                     VERT_SAVE_RGBA( 1 );
-                     VERT_SAVE_RGBA( 2 );
-		     VERT_COPY_RGBA1( v[0] );
-		     VERT_COPY_RGBA1( v[1] );
-		     VERT_COPY_RGBA1( v[2] );
-		  }
-		  VERT_SAVE_RGBA( 3 );
-		  VERT_COPY_RGBA1( v[3] );
-		  if (HAVE_SPEC) {
-                     if (!DO_FLAT) {
-                        VERT_SAVE_SPEC( 0 );
-                        VERT_SAVE_SPEC( 1 );
-                        VERT_SAVE_SPEC( 2 );
-			VERT_COPY_SPEC1( v[0] );
-			VERT_COPY_SPEC1( v[1] );
-			VERT_COPY_SPEC1( v[2] );
-		     }
-		     VERT_SAVE_SPEC( 3 );
-		     VERT_COPY_SPEC1( v[3] );
-		  }
+	    if (HAVE_BACK_COLORS) {
+	       if (!DO_FLAT) {
+		  VERT_SAVE_RGBA( 0 );
+		  VERT_SAVE_RGBA( 1 );
+		  VERT_SAVE_RGBA( 2 );
+		  VERT_COPY_RGBA1( v[0] );
+		  VERT_COPY_RGBA1( v[1] );
+		  VERT_COPY_RGBA1( v[2] );
 	       }
-	       else {
-	          if (!DO_FLAT) {
-		     VERT_SAVE_RGBA( 0 );
-		     VERT_SAVE_RGBA( 1 );
-		     VERT_SAVE_RGBA( 2 );
+	       VERT_SAVE_RGBA( 3 );
+	       VERT_COPY_RGBA1( v[3] );
+	       if (HAVE_SPEC) {
+		  if (!DO_FLAT) {
+		     VERT_SAVE_SPEC( 0 );
+		     VERT_SAVE_SPEC( 1 );
+		     VERT_SAVE_SPEC( 2 );
+		     VERT_COPY_SPEC1( v[0] );
+		     VERT_COPY_SPEC1( v[1] );
+		     VERT_COPY_SPEC1( v[2] );
 		  }
-	          VERT_SAVE_RGBA( 3 );
-
-		  if (VB->ColorPtr[1]->stride) {
-		     if (!DO_FLAT) {
-			VERT_SET_RGBA( v[0], vbcolor[e0] );
-			VERT_SET_RGBA( v[1], vbcolor[e1] );
-			VERT_SET_RGBA( v[2], vbcolor[e2] );
-		     }
-		     VERT_SET_RGBA( v[3], vbcolor[e3] );
-		  }
-		  else {
-		     if (!DO_FLAT) {
-			VERT_SET_RGBA( v[0], vbcolor[0] );
-			VERT_SET_RGBA( v[1], vbcolor[0] );
-			VERT_SET_RGBA( v[2], vbcolor[0] );
-		     }
-		     VERT_SET_RGBA( v[3], vbcolor[0] );
-		  }
-
-	          if (HAVE_SPEC && VB->SecondaryColorPtr[1]) {
-		     GLfloat (*vbspec)[4] = VB->SecondaryColorPtr[1]->data;
-		     ASSERT(VB->SecondaryColorPtr[1]->stride==4*sizeof(GLfloat));
-
-		     if (!DO_FLAT) {
-		        VERT_SAVE_SPEC( 0 );
-		        VERT_SAVE_SPEC( 1 );
-		        VERT_SAVE_SPEC( 2 );
-		        VERT_SET_SPEC( v[0], vbspec[e0] );
-		        VERT_SET_SPEC( v[1], vbspec[e1] );
-		        VERT_SET_SPEC( v[2], vbspec[e2] );
-		     }
-		     VERT_SAVE_SPEC( 3 );
-		     VERT_SET_SPEC( v[3], vbspec[e3] );
-	          }
+		  VERT_SAVE_SPEC( 3 );
+		  VERT_COPY_SPEC1( v[3] );
 	       }
 	    }
 	    else {
-	       GLfloat *vbindex = (GLfloat *)VB->IndexPtr[1]->data;
 	       if (!DO_FLAT) {
-		  VERT_SAVE_IND( 0 );
-		  VERT_SAVE_IND( 1 );
-		  VERT_SAVE_IND( 2 );
-		  VERT_SET_IND( v[0], vbindex[e0] );
-		  VERT_SET_IND( v[1], vbindex[e1] );
-		  VERT_SET_IND( v[2], vbindex[e2] );
+		  VERT_SAVE_RGBA( 0 );
+		  VERT_SAVE_RGBA( 1 );
+		  VERT_SAVE_RGBA( 2 );
+	       }
+	       VERT_SAVE_RGBA( 3 );
+
+	       if (VB->BackfaceColorPtr->stride) {
+		  if (!DO_FLAT) {
+		     VERT_SET_RGBA( v[0], vbcolor[e0] );
+		     VERT_SET_RGBA( v[1], vbcolor[e1] );
+		     VERT_SET_RGBA( v[2], vbcolor[e2] );
+		  }
+		  VERT_SET_RGBA( v[3], vbcolor[e3] );
+	       }
+	       else {
+		  if (!DO_FLAT) {
+		     VERT_SET_RGBA( v[0], vbcolor[0] );
+		     VERT_SET_RGBA( v[1], vbcolor[0] );
+		     VERT_SET_RGBA( v[2], vbcolor[0] );
+		  }
+		  VERT_SET_RGBA( v[3], vbcolor[0] );
+	       }
+
+	       if (HAVE_SPEC && VB->BackfaceSecondaryColorPtr) {
+		  GLfloat (*vbspec)[4] = VB->BackfaceSecondaryColorPtr->data;
+		  ASSERT(VB->BackfaceSecondaryColorPtr->stride==4*sizeof(GLfloat));
+
+		  if (!DO_FLAT) {
+		     VERT_SAVE_SPEC( 0 );
+		     VERT_SAVE_SPEC( 1 );
+		     VERT_SAVE_SPEC( 2 );
+		     VERT_SET_SPEC( v[0], vbspec[e0] );
+		     VERT_SET_SPEC( v[1], vbspec[e1] );
+		     VERT_SET_SPEC( v[2], vbspec[e2] );
+		  }
+		  VERT_SAVE_SPEC( 3 );
+		  VERT_SET_SPEC( v[3], vbspec[e3] );
 	       }
-	       VERT_SAVE_IND( 3 );
-	       VERT_SET_IND( v[3], vbindex[e3] );
 	    }
 	 }
       }
@@ -539,21 +469,20 @@ static void TAG(quad)( GLcontext *ctx,
 	    GLfloat bc	= b * ic;
 	    if ( ac < 0.0f ) ac = -ac;
 	    if ( bc < 0.0f ) bc = -bc;
-	    offset += MAX2( ac, bc ) * ctx->Polygon.OffsetFactor;
+	    offset += MAX2( ac, bc ) * ctx->Polygon.OffsetFactor / ctx->DrawBuffer->_MRD;
 	 }
 	 offset *= ctx->DrawBuffer->_MRD * (REVERSE_DEPTH ? -1.0 : 1.0);
       }
    }
 
    if (DO_FLAT) {
-      if (HAVE_RGBA) {
 	 VERT_SAVE_RGBA( 0 );
 	 VERT_SAVE_RGBA( 1 );
 	 VERT_SAVE_RGBA( 2 );
 	 VERT_COPY_RGBA( v[0], v[3] );
 	 VERT_COPY_RGBA( v[1], v[3] );
 	 VERT_COPY_RGBA( v[2], v[3] );
-	 if (HAVE_SPEC && VB->SecondaryColorPtr[0]) {
+	 if (HAVE_SPEC && VB->AttribPtr[_TNL_ATTRIB_COLOR1]) {
 	    VERT_SAVE_SPEC( 0 );
 	    VERT_SAVE_SPEC( 1 );
 	    VERT_SAVE_SPEC( 2 );
@@ -561,15 +490,6 @@ static void TAG(quad)( GLcontext *ctx,
 	    VERT_COPY_SPEC( v[1], v[3] );
 	    VERT_COPY_SPEC( v[2], v[3] );
 	 }
-      }
-      else {
-	 VERT_SAVE_IND( 0 );
-	 VERT_SAVE_IND( 1 );
-	 VERT_SAVE_IND( 2 );
-	 VERT_COPY_IND( v[0], v[3] );
-	 VERT_COPY_IND( v[1], v[3] );
-	 VERT_COPY_IND( v[2], v[3] );
-      }
    }
 
    if (mode == GL_POINT) {
@@ -625,55 +545,37 @@ static void TAG(quad)( GLcontext *ctx,
       VERT_SET_Z(v[3], z[3]);
    }
 
-   if (DO_TWOSIDE && facing == 1)
-   {
-      if (HAVE_RGBA) {
-	 if (!DO_FLAT) {
-	    VERT_RESTORE_RGBA( 0 );
-	    VERT_RESTORE_RGBA( 1 );
-	    VERT_RESTORE_RGBA( 2 );
-	 }
-	 VERT_RESTORE_RGBA( 3 );
-	 if (HAVE_SPEC) {
-	    if (!DO_FLAT) {
-	       VERT_RESTORE_SPEC( 0 );
-	       VERT_RESTORE_SPEC( 1 );
-	       VERT_RESTORE_SPEC( 2 );
-	    }
-	    VERT_RESTORE_SPEC( 3 );
-	 }
+   if (DO_TWOSIDE && facing == 1) {
+      if (!DO_FLAT) {
+	 VERT_RESTORE_RGBA( 0 );
+	 VERT_RESTORE_RGBA( 1 );
+	 VERT_RESTORE_RGBA( 2 );
       }
-      else {
+      VERT_RESTORE_RGBA( 3 );
+      if (HAVE_SPEC) {
 	 if (!DO_FLAT) {
-	    VERT_RESTORE_IND( 0 );
-	    VERT_RESTORE_IND( 1 );
-	    VERT_RESTORE_IND( 2 );
+	    VERT_RESTORE_SPEC( 0 );
+	    VERT_RESTORE_SPEC( 1 );
+	    VERT_RESTORE_SPEC( 2 );
 	 }
-	 VERT_RESTORE_IND( 3 );
+	 VERT_RESTORE_SPEC( 3 );
       }
    }
 
 
    if (DO_FLAT) {
-      if (HAVE_RGBA) {
 	 VERT_RESTORE_RGBA( 0 );
 	 VERT_RESTORE_RGBA( 1 );
 	 VERT_RESTORE_RGBA( 2 );
-	 if (HAVE_SPEC && VB->SecondaryColorPtr[0]) {
+	 if (HAVE_SPEC && VB->AttribPtr[_TNL_ATTRIB_COLOR1]) {
 	    VERT_RESTORE_SPEC( 0 );
 	    VERT_RESTORE_SPEC( 1 );
 	    VERT_RESTORE_SPEC( 2 );
 	 }
-      }
-      else {
-	 VERT_RESTORE_IND( 0 );
-	 VERT_RESTORE_IND( 1 );
-	 VERT_RESTORE_IND( 2 );
-      }
    }
 }
 #else
-static void TAG(quad)( GLcontext *ctx, GLuint e0,
+static void TAG(quadr)( GLcontext *ctx, GLuint e0,
 		       GLuint e1, GLuint e2, GLuint e3 )
 {
    if (DO_UNFILLED) {
@@ -705,33 +607,22 @@ static void TAG(line)( GLcontext *ctx, GLuint e0, GLuint e1 )
    v[1] = (VERTEX *)GET_VERTEX(e1);
 
    if (DO_FLAT) {
-      if (HAVE_RGBA) {
 	 VERT_SAVE_RGBA( 0 );
 	 VERT_COPY_RGBA( v[0], v[1] );
-	 if (HAVE_SPEC && VB->SecondaryColorPtr[0]) {
+	 if (HAVE_SPEC && VB->AttribPtr[_TNL_ATTRIB_COLOR1]) {
 	    VERT_SAVE_SPEC( 0 );
 	    VERT_COPY_SPEC( v[0], v[1] );
 	 }
-      }
-      else {
-	 VERT_SAVE_IND( 0 );
-	 VERT_COPY_IND( v[0], v[1] );
-      }
    }
 
    LINE( v[0], v[1] );
 
    if (DO_FLAT) {
-      if (HAVE_RGBA) {
 	 VERT_RESTORE_RGBA( 0 );
 
-	 if (HAVE_SPEC && VB->SecondaryColorPtr[0]) {
+	 if (HAVE_SPEC && VB->AttribPtr[_TNL_ATTRIB_COLOR1]) {
 	    VERT_RESTORE_SPEC( 0 );
 	 }
-      }
-      else {
-	 VERT_RESTORE_IND( 0 );
-      }
    }
 }
 #endif
@@ -765,7 +656,7 @@ static void TAG(points)( GLcontext *ctx, GLuint first, GLuint last )
 static void TAG(init)( void )
 {
 #if DO_QUAD
-   TAB[IND].quad = TAG(quad);
+   TAB[IND].quad = TAG(quadr);
 #endif
 #if DO_TRI
    TAB[IND].triangle = TAG(triangle);
@@ -781,23 +672,9 @@ static void TAG(init)( void )
 #undef IND
 #undef TAG
 
-#if HAVE_RGBA
-#undef VERT_SET_IND
-#undef VERT_COPY_IND
-#undef VERT_SAVE_IND
-#undef VERT_RESTORE_IND
 #if HAVE_BACK_COLORS
 #undef VERT_SET_RGBA
 #endif
-#else
-#undef VERT_SET_RGBA
-#undef VERT_COPY_RGBA
-#undef VERT_SAVE_RGBA
-#undef VERT_RESTORE_RGBA
-#if HAVE_BACK_COLORS
-#undef VERT_SET_IND
-#endif
-#endif
 
 #if !HAVE_SPEC
 #undef VERT_SET_SPEC
@@ -815,7 +692,6 @@ static void TAG(init)( void )
 
 #if !HAVE_BACK_COLORS
 #undef VERT_COPY_SPEC1
-#undef VERT_COPY_IND1
 #undef VERT_COPY_RGBA1
 #endif
 
